/********************************************************************
* JREE Bounding Code
*
* This code runs the bounds routine on experimental data with 
* defined principal strata. Please see the JREE article for specifics.
*
* This code is meant to be used as a template and has not been implemented
* as a stand alone routine. You will have to alter the code to fit your application. 
* 
* Contact: Jane Furey (jlfurey@umich.edu)
**********************************************************************/

/***************************************************************
Fill in Marcos and set seed
***************************************************************/

set seed /*choose a seed */
global indir /*your data directory */
global outdir 
local indata /* your input dataset */ 
local outdata /* your output dataset*/

cd $dir

use "`indata'", clear

/***********************************************************************************************
STEP 1: Define key variables: 

1. Outcome = outcome of interest (is a dichotomous outcome in this case -- a 
		continuous outcome would also work, but the outputting code should be adjusted)
2. Treatment indicator (should be 0 for control and 1 for treatment assignment
2. Strata variable = indicates posible strata (here the strata variable is numbered 1, 2, 3)
		**Code can be adopted to calculate bounds for more or less strata)
		
For your reference, our variables are as follows:
	outcome = (0 = not on track in 9th grade) (1 = on track in 9th grade)
	strata = (1 = enrolled in ECHS) (2 = Enrolled in Low Quality HS) (3 = Enrolled in High Quality HS)
	treatment = (0 = assigned to control) (1 = assigned to treatment)
		
***********************************************************************************************/

gen treatment = /* your treatment variable */
gen outcome = /* your outcome variable */ 
gen strata = /*your strata variable */

/*************************************************************************************
STEP 2: Determine the proportion of T/C cases in each strata
	This code would create the proprotions presented in Table 1 of the JREE article
**************************************************************************************/ 

qui sum strata
local smin = r(min)
local smax = r(max)

forvalues t = 0/1 {
	count if treatment == `t'
	local `t'tot = r(N)
		forvalues s = `smin'/`smax' {
			qui count if treatment == `t' & strata == `s' 
				local p_tc`t'_s`s' = `r(N)'/``t'tot'
				local n_tc`t'_s`s' = `r(N)'
				
	} 
	}
	
/*************************************************************************************
STEP 3: Calculate observed outcomes for each treatment/strata combination
	
**************************************************************************************/
forvalues t = 0/1 {
forvalues s = `smin'/`smax' {
	qui sum outcome if treatment == `t' & strata == `s' 
		local y_tc`t'_s`s' = r(mean)
	} 
	}
	
/******************************************************************************************
STEP 4: Use observed proportions to determine the proportions of active strata 
		within the PS matrix (see Table 2 in the JREE article for guidance)
		
	*Note: THIS SECTION OF CODE SHOULD BE ALTERED TO FIT YOUR PRINCIPAL STRATA
				You may make different assumptions about which strata to include/exclude,
				so you may need to define quantities and outcomes differently.
				
				This code maps to the analyses in the JREE article and should be used 
				as a reference! 
*******************************************************************************************/

*Cell Numbers refer to Table 2 in the JREE article*

**Always takers: treatment (Cell (1))
local p_at_t = `p_tc0_s1'
local n_at_t = `n_tc0_s1'

**Always takers: low quality HS (Cell (5))
local p_at_lq = `p_tc1_s2'
local n_at_lq = `n_tc1_s2'

**Always taker: high quality HS (Cell (9))
local p_at_hq = `p_tc1_s3'
local n_at_hq = `n_tc1_s3'

**Compliers: Low quality HS (Cell (2))
local p_com_lq = `p_tc0_s2' - `p_tc1_s2'
local n_com_lq = `n_tc0_s2' - `n_tc1_s2'

**Compliers: High Quality HS (Cell (3))
local p_com_hq = `p_tc0_s3' - `p_tc1_s3'	
local n_com_hq = `n_tc0_s3' - `n_tc1_s3'

***********************************************
*Check: Proportions should sum to 1
***********************************************

di `p_at_t' + `p_at_lq' + `p_at_hq' + `p_com_lq' + `p_com_hq'

************************************************************************************************

/******************************************************************************************
STEP 5: Use marginal outcome values to determine the outcomes for observable cells
		
	*Note: THIS SECTION OF CODE SHOULD BE ALTERED TO FIT YOUR PRINCIPAL STRATA
				You may make different assumptions about which strata to include/exclude,
				so you may need to calculate outcomes differently.
				
				This code maps to the analyses in the JREE article and should be used 
				as a reference! 
*******************************************************************************************/

**Always takers: treatment (Cell (1))
local y_at_t = `y_tc0_s1'

**Always takers: low quality HS (Cell (5))
local y_at_lq = `y_tc1_s2'

**Always taker: high quality HS (Cell (9))
local y_at_hq = `y_tc1_s3'

**For compliers, use marginal outcome means as a weighted average of means of AT and Compliers
	*We are solving for the means under control
**Compliers: Low quality HS (Cell (2))
local y_com_lq = (`p_tc0_s2'*`y_tc0_s2' - `p_at_lq'*`y_at_lq')/`p_com_lq'

**Compliers: High Quality HS (Cell (3))
local y_com_hq = (`p_tc0_s3'*`y_tc0_s3' - `p_at_hq'*`y_at_hq')/`p_com_hq'	


/****************************************************************************************************
STEP 6: Calculate Bounds for Complier Cells
	The means for Cell (2) and Cell (3) [JREE Table 2] will average out to the marginal value
		the marginal value represents the mean of the treatment compliers (treatment = 1 & strata = 1)
******************************************************************************************************/

*Marginal value (mean outcome for treatment compliers)
local marg_y1 = `y_tc1_s1'*`p_tc1_s1'

*First take out the treatment always takers:
local marg_y2 = `marg_y1' - `y_at_t'*`p_at_t'

*Bounds for low quanity HS compliers (Cell (2)):
*For upper bound, assume y of high quality compliers is 0
local y_lq_upper = (`marg_y2' - `p_com_hq'*0)/`p_com_lq'

*for lower bound, asusme y of high quality compliers is 1
local y_lq_lower = (`marg_y2' - `p_com_hq'*1)/`p_com_lq'

*Bounds for low quanity HS compliers (Cell (3)):
*For upper bound, assume y of low quality compliers is 0
local y_hq_upper = (`marg_y2' - `p_com_lq'*0)/`p_com_hq'
	
*for lower bound, asusme y of low quality compliers is 1
local y_hq_lower = (`marg_y2' - `p_com_lq'*1)/`p_com_hq'

*Reset bounds that are above or below 1 and 0
foreach val in hq lq {
	if `y_`val'_upper' > 1 {
		local y_`val'_upper = 1
	}
	
	if `y_`val'_lower' < 0 {
		local y_`val'_lower = 0
	}
}


/*************************************************************************
STEP 7: Create Table 3 from JREE Article
**************************************************************************/

clear
gen stratum = ""
gen p = .
gen n = .
gen y_c = .
gen y_t_lower = .
gen y_t_upper = .
gen ITT_lower = .
gen ITT_upper = .

local strata at_t at_hq com_hq at_lq com_lq
local newobs = 1 
foreach val of local strata {
	set obs `newobs'
	replace stratum = "`val'" if _n == _N
	replace p = `p_`val'' if _n == _N
	replace n = `n_`val'' if _n == _N	
	replace y_c = `y_`val'' if _n == _N 
	local newobs = `newobs' + 1 
	}
	
local at at_t at_hq	at_lq
foreach val of local at {
	replace y_t_upper = `y_`val'' if stratum == "`val'"
	replace y_t_lower = `y_`val''  if stratum == "`val'"
	}
	
local com lq hq 
foreach val of local com {
	replace y_t_upper = `y_`val'_upper' if stratum == "com_`val'"
	replace y_t_lower = `y_`val'_lower'   if stratum == "com_`val'"
	}
	
replace ITT_upper = y_t_upper - y_c
replace ITT_lower = y_t_lower - y_c

cd $outdir
save "`outdata'", replace
